<?php

declare(strict_types=1);

namespace Dedi\SyliusStockHistoryPlugin\Resolver\Order\Api;

use Dedi\SyliusStockHistoryPlugin\Enum\InventoryLogEntryType;
use Dedi\SyliusStockHistoryPlugin\Resolver\Order\InventoryLogOrderResolverInterface;
use Sylius\Component\Core\Model\OrderInterface;
use Sylius\Component\Core\Repository\OrderRepositoryInterface;
use Symfony\Component\HttpFoundation\RequestStack;

class OrderEditResolver implements InventoryLogOrderResolverInterface
{
    public function __construct(
        private readonly RequestStack $requestStack,
        private readonly OrderRepositoryInterface $orderRepository,
    ) {
    }

    public function resolve(string $type): ?OrderInterface
    {
        if (!$this->supports($type)) {
            return null;
        }

        $request = $this->requestStack->getCurrentRequest();

        if (null === $request) {
            return null;
        }

        $path = $request->getPathInfo();
        $orderToken = null;

        if ($type === InventoryLogEntryType::API_ORDER_COMPLETE->value && 1 === preg_match('/^\/api\/v[0-9]+\/(shop|admin)\/orders\/([a-zA-Z0-9_-]+)\/(complete|cancel)$/', $path, $matches)) {
            $orderToken = $matches[2];
        }

        if (null !== $orderToken) {
            /** @var ?OrderInterface $order */
            $order = $this->orderRepository->findOneBy(['tokenValue' => $orderToken]);

            return $order;
        }

        return null;
    }

    public function supports(string $type): bool
    {
        return $type === InventoryLogEntryType::API_ORDER_COMPLETE->value || $type === InventoryLogEntryType::API_ORDER_CANCEL->value;
    }
}
